clc; 
clear all
close all 

global V_wind Rho Wing_surface Mean_chord l_4 l_3 Cl_alpha Cl_zero Cm_alpha ...
        Cm_zero Cd_zero K_cd Mass
    
%% INPUT VAR

Mass = 2.35; %[Kg] Mass of the vehicle
V_wind = 5; %[m/s] Wind speed     
Rho = 1.22; %[kg/m^3] Air density
Wing_surface = 0.57; %[m^2] Wing surface
Mean_chord = 0.3325; %[m] Average wing chord
l_4 = 0.29; %[m] x-axis offset of the front motors
l_3 = 0.36; %[m] x-axis offset of the back motors
Cl_alpha = 4.2; %[1/rad] Cl rate in 1/rad
Cl_zero = -0.05; % Cl at alpha equal to zero
Cm_alpha = -0.58; %[1/rad] Cm rate in 1/rad
Cm_zero = .002; % Cm at alpha equal to zero
Cd_zero = 0.5; % Cd at alpha equal to zero
K_cd = 0.08; %Quadratic therm of drag
M_q = 0; %Coefficient for torque vs q

%Actuator properties:
Max_thrust = 8.8; %[N] Maximum thrust value
k_t = 0.95*10^-5; %Coefficient of thrust for the motor
Azimuth_cut_freq = 30.7; %[rad/s] Cutting frequency of the first order azimuth tilting dynamics
Elevation_cut_freq = 26.16; %[rad/s] Cutting frequency of the first order elevation tilting dynamics
Servo_delay_Ts = 15;%[mS] Delay of the servomotor activating the tilting mechanism
Motor_cut_freq = 19.35; %[rad/s] Cutting frequency of the first order motor dynamics
Motor_delay_Ts = 15;%[mS] Delay of the motor
I_yy = 0.15; %[kg*m^2] Intertia of the vehicle in the pitch axes

%% Script
%Calculation of the omega equilibrium
V_test = V_wind;
V_wind = 0;
fun = @system_equation_quadplane_conventional;
x0 = [Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,-1];
x = fsolve(fun,x0);

Omega_eq = sqrt(x(1:4)/k_t);
Thrust_eq = x(1:4);

max_omega_motor = sqrt(Max_thrust/k_t); 

%Conventional platform 
V_wind = V_test;
fun = @system_equation_quadplane_conventional;
x0 = [Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,-1];
x = fsolve(fun,x0);
theta_eq = x(5);
fprintf('/n Theta equilibrium = %d deg/n',theta_eq*57)
Omega_wind_conventional = sqrt(x(1:4)/k_t);
Thrust_wind_conventional = x(1:4);

%Overactuated platform 
fun = @system_equation_quadplane_overactuated;
x0 = [Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,-1];
x = fsolve(fun,x0);

Omega_wind_overactuated = sqrt(x(1:4)/k_t);
Thrust_wind_overactuated = x(1:4);
b_eq = x(5);


%% Setting up the simulink model
Simulation_runtime = 15; %[s] decide the simulation runtime
Wind_omega_for_step = 0;
Step_time = 5; %[s] decide when to have the gust
Step_magnetude = -V_wind; %[m/s] decide the gust strength

act_eff = 0.1283; % Actuator effectiveness of motors calculated from the linearization point
act_eff_tilt = 9.81; % Actuator effectiveness of tilt calculated from the linearization point
M_alpha = 0.5 * Rho * Wing_surface * V_wind^2 * Cm_alpha * Mean_chord; %Computation of the stability derivative

%Run simulation: 
Conventional_results = sim('pitch_analysis_conventional_simplified',Simulation_runtime);
Overactuated_results = sim('pitch_analysis_overactuated_simplified',Simulation_runtime);
timescale_conv = Conventional_results.Vx.time;
timescale_over = Overactuated_results.Vx.time;
idx_conv = timescale_conv>=Step_time-.5;
idx_over = timescale_over>=Step_time-.5;

%Plot 
figure()
plot(timescale_conv(idx_conv) - Step_time,Conventional_results.Vx.signals.values(idx_conv),'r')
title('Linear x-earth speed evolution')
hold on 
plot(timescale_over(idx_over) - Step_time,Overactuated_results.Vx.signals.values(idx_over),'b')
legend('Conventional speed','overactuated speed');
xlabel('Time [s]');
ylabel('x-earth speed [m/s]');
grid on
xlim([-.5 3]);

figure()
plot(timescale_conv(idx_conv) - Step_time,Conventional_results.Ax.signals.values(idx_conv),'r')
title('Linear x-earth acceleration evolution')
hold on 
plot(timescale_over(idx_over) - Step_time,Overactuated_results.Ax.signals.values(idx_over),'b')
legend('Conventional acceleration','overactuated acceleration');
xlabel('Time [s]');
ylabel('x-earth acceleration [m/s^2]');
grid on
xlim([-.5 3]);

figure()
plot(timescale_conv(idx_conv) - Step_time,Conventional_results.x_earth.signals.values(idx_conv),'r')
title('X-earth position evolution')
hold on 
plot(timescale_over(idx_over) - Step_time,Overactuated_results.x_earth.signals.values(idx_over),'b')
legend('Conventional position','overactuated position');
xlabel('Time [s]');
ylabel('x-earth position [m]');
grid on
xlim([-.5 3]);

figure()
plot(timescale_conv(idx_conv) - Step_time,Conventional_results.Theta.signals.values(idx_conv),'r')
title('Tilting elevator and theta angle evolution')
hold on 
plot(timescale_over(idx_over) - Step_time,Overactuated_results.Tilt_el.signals.values(idx_over),'b')
legend('Conventional theta angle','overactuated elevation tilt angle');
xlabel('Time [s]');
ylabel('Angle [deg]');
grid on
xlim([-.5 3]);
