clc; 
clear all
close all 

global V_wind Rho Wing_surface Mean_chord l_4 l_3 Cl_alpha Cl_zero Cm_alpha ...
        Cm_zero Cd_zero K_cd Mass
    
%% INPUT VAR

Mass = 2.35; %[Kg] Mass of the vehicle
V_wind = 5; %[m/s] Wind speed     
Rho = 1.22; %[kg/m^3] Air density
Wing_surface = 0.57; %[m^2] Wing surface
Mean_chord = 0.3325; %[m] Average wing chord
l_4 = 0.29; %[m] x-axis offset of the front motors
l_3 = 0.36; %[m] x-axis offset of the back motors
Cl_alpha = 4.2; %[1/rad] Cl rate in 1/rad
Cl_zero = -0.05; % Cl at alpha equal to zero
Cm_alpha = -0.58; %[1/rad] Cm rate in 1/rad
Cm_zero = .002; % Cm at alpha equal to zero
Cd_zero = 0.55; % Cd at alpha equal to zero
K_cd = 0.08; %Quadratic therm of drag
M_q = 0; %Coefficient for torque vs q

%Actuator properties:
Max_thrust = 8.8; %[N] Maximum thrust value
k_t = 0.95*10^-5; %Coefficient of thrust for the motor
Azimuth_cut_freq = 30.7; %[rad/s] Cutting frequency of the first order azimuth tilting dynamics
Elevation_cut_freq = 26.16; %[rad/s] Cutting frequency of the first order elevation tilting dynamics
Servo_delay_Ts = 15;%[mS] Delay of the servomotor activating the tilting mechanism
Motor_cut_freq = 19.35; %[rad/s] Cutting frequency of the first order motor dynamics
Motor_delay_Ts = 15;%[mS] Delay of the motor
I_yy = 0.15; %[kg*m^2] Intertia of the vehicle in the pitch axes

%% Script
%Calculation of the omega equilibrium
V_test = V_wind;
V_wind = 0;
fun = @system_equation_quadplane_conventional;
x0 = [Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,-1];
x = fsolve(fun,x0);

Omega_eq = sqrt(x(1:4)/k_t);
Thrust_eq = x(1:4);

max_omega_motor = sqrt(Max_thrust/k_t); 

%Conventional platform 
V_wind = V_test;
fun = @system_equation_quadplane_conventional;
x0 = [Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,-1];
x = fsolve(fun,x0);
theta_eq = x(5);
fprintf('/n Theta equilibrium = %d deg/n',theta_eq*57)
Omega_wind_conventional = sqrt(x(1:4)/k_t);
Thrust_wind_conventional = x(1:4);

%Overactuated platform 
fun = @system_equation_quadplane_overactuated;
x0 = [Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,Mass*9.81/4,-1];
x = fsolve(fun,x0);

Omega_wind_overactuated = sqrt(x(1:4)/k_t);
Thrust_wind_overactuated = x(1:4);
b_eq = x(5);


%% Setting up the simulink model
Simulation_runtime = 15; %[s] decide the simulation runtime
Wind_omega_for_step = 0;
Step_time = 5; %[s] decide when to have the gust
% Step_magnetude = -V_wind; %[m/s] decide the gust strength
Step_magnetude = 0; %[m/s] decide the gust strength

act_eff = 0.1283; % Actuator effectiveness of motors calculated from the linearized model
act_eff_tilt = 9.81; % Actuator effectiveness of tilt calculated from the linearized model
M_alpha = 0.5 * Rho * Wing_surface * V_wind^2 * Cm_alpha * Mean_chord; %Computation of the stability derivative

%Run simulation: 
Conventional_results = sim('full_dof_model_conventional',Simulation_runtime);
Overactuated_results = sim('full_dof_model_overactuated',Simulation_runtime);
timescale_conv = Conventional_results.Vx.time;
timescale_over = Overactuated_results.Vx.time;
idx_conv = timescale_conv>=Step_time-.5;
idx_over = timescale_over>=Step_time-.5;

%Calculate the max. acceleration time and the overall displacement
if(Step_magnetude == 0) 
    max_acc_time_over = Overactuated_results.Ax.time( Overactuated_results.Ax.signals.values == max(Overactuated_results.Ax.signals.values) );
    max_acc_time_conv = Conventional_results.Ax.time( Conventional_results.Ax.signals.values == max(Conventional_results.Ax.signals.values) );
    fprintf('\n \nMax. acceleration time overactuated platform = %f [s]\n',max_acc_time_over - 5);
    fprintf('Max. acceleration time conventional platform = %f [s]\n',max_acc_time_conv - 5);
    fprintf('Time offset between overactuated and conventional acceleration = %f [s]\n', (max_acc_time_conv - max_acc_time_over));
end
if(Step_magnetude ~= 0) 
    fprintf('\n \nMax. displacement overactuated platform = %f [m]\n',abs(min(Overactuated_results.x_earth.signals.values)));
    fprintf('Max. displacement conventional platform = %f [m]\n',abs(min(Conventional_results.x_earth.signals.values)));   
end

%Plot 
figure()
plot(timescale_conv(idx_conv) - Step_time,Conventional_results.Vx.signals.values(idx_conv),'r','LineWidth',1)
title('Linear x-earth speed evolution')
hold on 
plot(timescale_over(idx_over) - Step_time,Overactuated_results.Vx.signals.values(idx_over),'b','LineWidth',1)
legend_speed = legend('Conventional vehicle','overactuated vehicle','Interpreter','latex');
label_x_speed = xlabel('Time [s]');
label_y_speed = ylabel('x-earth speed [m/s]');
set(legend_speed,'FontSize',13);
set(label_y_speed,'FontSize',17);
set(label_x_speed,'FontSize',17);
grid on
xlim([-.5 3]);

figure()
plot(timescale_conv(idx_conv) - Step_time,Conventional_results.Ax.signals.values(idx_conv),'r','LineWidth',1)
title('Linear x-earth acceleration evolution')
hold on 
plot(timescale_over(idx_over) - Step_time,Overactuated_results.Ax.signals.values(idx_over),'b','LineWidth',1)
legend_acceleration = legend('Conventional vehicle','overactuated vehicle','Interpreter','latex');
label_x_acceleration = xlabel('Time [s]','Interpreter','latex');
label_y_acceleration = ylabel('x-earth acceleration [$\frac{m}{s^2}$]','Interpreter','latex');
set(legend_acceleration,'FontSize',13);
set(label_y_acceleration,'FontSize',17);
set(label_x_acceleration,'FontSize',17);
grid on
xlim([-.5 2.5]);

figure()
plot(timescale_conv(idx_conv) - Step_time,Conventional_results.x_earth.signals.values(idx_conv),'r','LineWidth',1)
title('X-earth position evolution')
hold on 
plot(timescale_over(idx_over) - Step_time,Overactuated_results.x_earth.signals.values(idx_over),'b','LineWidth',1)
legend_position = legend('Conventional vehicle','overactuated vehicle','Interpreter','latex');
label_x_position = xlabel('Time [s]','Interpreter','latex');
label_y_position = ylabel('x-earth position [m]','Interpreter','latex');
set(legend_position,'FontSize',13);
set(label_y_position,'FontSize',17);
set(label_x_position,'FontSize',17);
grid on
xlim([-.5 6]);

figure()
plot(timescale_conv(idx_conv) - Step_time,Conventional_results.Theta.signals.values(idx_conv),'r','LineWidth',1)
title('Tilting elevator $b_{tot}$ and $\theta$ angle evolution','Interpreter','latex')
hold on 
plot(timescale_over(idx_over) - Step_time,Overactuated_results.Tilt_el.signals.values(idx_over),'b','LineWidth',1)
legend_angles = legend('Conventional $\theta$ angle','Overactuated $b_{tot}$ angle','Interpreter','latex');
label_x_angle = xlabel('Time [s]','Interpreter','latex');
label_y_angle = ylabel('Angle [deg]','Interpreter','latex');
set(legend_angles,'FontSize',13);
set(label_y_angle,'FontSize',17);
set(label_x_angle,'FontSize',17);
grid on
xlim([-.5 2.5]);
